class AndBP1D:
    pass


class AndUP1D:
    pass


class ContourUP1D:
    '''Class hierarchy: freestyle.types.UnaryPredicate1D > ContourUP1D '''

    def __call__(self, inter):
        '''Returns true if the Interface1D is a contour. An Interface1D is a contour if it is borded by a different shape on each of its sides. 

        :param inter: An Interface1D object. 
        :type inter: freestyle.types.Interface1D
        :rtype: bool 
        :return:  True if the Interface1D is a contour, false otherwise. 
        '''
        pass


class DensityLowerThanUP1D:
    '''Class hierarchy: freestyle.types.UnaryPredicate1D > DensityLowerThanUP1D '''

    def __init__(self, threshold, sigma=2.0):
        '''Builds a DensityLowerThanUP1D object. 

        :param threshold: The value of the threshold density. Any Interface1D having a density lower than this threshold will match. 
        :type threshold: float
        :param sigma: The sigma value defining the density evaluation window size used in the freestyle.functions.DensityF0D functor. 
        :type sigma: float
        '''
        pass

    def __call__(self, inter):
        '''Returns true if the density evaluated for the Interface1D is less than a user-defined density value. 

        :param inter: An Interface1D object. 
        :type inter: freestyle.types.Interface1D
        :rtype: bool 
        :return:  True if the density is lower than a threshold. 
        '''
        pass


class EqualToChainingTimeStampUP1D:
    '''Class hierarchy: freestyle.types.UnaryPredicate1D > freestyle.types.EqualToChainingTimeStampUP1D '''

    def __init__(self, ts):
        '''Builds a EqualToChainingTimeStampUP1D object. 

        :param ts: A time stamp value. 
        :type ts: int
        '''
        pass

    def __call__(self, inter):
        '''Returns true if the Interface1D’s time stamp is equal to a certain user-defined value. 

        :param inter: An Interface1D object. 
        :type inter: freestyle.types.Interface1D
        :rtype: bool 
        :return:  True if the time stamp is equal to a user-defined value. 
        '''
        pass


class EqualToTimeStampUP1D:
    '''Class hierarchy: freestyle.types.UnaryPredicate1D > EqualToTimeStampUP1D '''

    def __init__(self, ts):
        '''Builds a EqualToTimeStampUP1D object. 

        :param ts: A time stamp value. 
        :type ts: int
        '''
        pass

    def __call__(self, inter):
        '''Returns true if the Interface1D’s time stamp is equal to a certain user-defined value. 

        :param inter: An Interface1D object. 
        :type inter: freestyle.types.Interface1D
        :rtype: bool 
        :return:  True if the time stamp is equal to a user-defined value. 
        '''
        pass


class ExternalContourUP1D:
    '''Class hierarchy: freestyle.types.UnaryPredicate1D > ExternalContourUP1D '''

    def __call__(self, inter):
        '''Returns true if the Interface1D is an external contour. An Interface1D is an external contour if it is borded by no shape on one of its sides. 

        :param inter: An Interface1D object. 
        :type inter: freestyle.types.Interface1D
        :rtype: bool 
        :return:  True if the Interface1D is an external contour, false otherwise. 
        '''
        pass


class FalseBP1D:
    '''Class hierarchy: freestyle.types.BinaryPredicate1D > FalseBP1D '''

    def __call__(self, inter1, inter2):
        '''Always returns false. 

        :param inter1: The first Interface1D object. 
        :type inter1: freestyle.types.Interface1D
        :param inter2: The second Interface1D object. 
        :type inter2: freestyle.types.Interface1D
        :rtype: bool 
        :return:  False. 
        '''
        pass


class FalseUP0D:
    '''Class hierarchy: freestyle.types.UnaryPredicate0D > FalseUP0D '''

    def __call__(self, it):
        '''Always returns false. 

        :param it: An Interface0DIterator object. 
        :type it: freestyle.types.Interface0DIterator
        :rtype: bool 
        :return:  False. 
        '''
        pass


class FalseUP1D:
    '''Class hierarchy: freestyle.types.UnaryPredicate1D > FalseUP1D '''

    def __call__(self, inter):
        '''Always returns false. 

        :param inter: An Interface1D object. 
        :type inter: freestyle.types.Interface1D
        :rtype: bool 
        :return:  False. 
        '''
        pass


class Length2DBP1D:
    '''Class hierarchy: freestyle.types.BinaryPredicate1D > Length2DBP1D '''

    def __call__(self, inter1, inter2):
        '''Returns true if the 2D length of inter1 is less than the 2D length of inter2. 

        :param inter1: The first Interface1D object. 
        :type inter1: freestyle.types.Interface1D
        :param inter2: The second Interface1D object. 
        :type inter2: freestyle.types.Interface1D
        :rtype: bool 
        :return:  True or false. 
        '''
        pass


class MaterialBP1D:
    '''Checks whether the two supplied ViewEdges have the same material. '''

    pass


class NotBP1D:
    pass


class NotUP1D:
    pass


class ObjectNamesUP1D:
    pass


class OrBP1D:
    pass


class OrUP1D:
    pass


class QuantitativeInvisibilityRangeUP1D:
    pass


class QuantitativeInvisibilityUP1D:
    '''Class hierarchy: freestyle.types.UnaryPredicate1D > QuantitativeInvisibilityUP1D '''

    def __init__(self, qi=0):
        '''Builds a QuantitativeInvisibilityUP1D object. 

        :param qi: The Quantitative Invisibility you want the Interface1D to have. 
        :type qi: int
        '''
        pass

    def __call__(self, inter):
        '''Returns true if the Quantitative Invisibility evaluated at an Interface1D, using the freestyle.functions.QuantitativeInvisibilityF1D functor, equals a certain user-defined value. 

        :param inter: An Interface1D object. 
        :type inter: freestyle.types.Interface1D
        :rtype: bool 
        :return:  True if Quantitative Invisibility equals a user-defined value. 
        '''
        pass


class SameShapeIdBP1D:
    '''Class hierarchy: freestyle.types.BinaryPredicate1D > SameShapeIdBP1D '''

    def __call__(self, inter1, inter2):
        '''Returns true if inter1 and inter2 belong to the same shape. 

        :param inter1: The first Interface1D object. 
        :type inter1: freestyle.types.Interface1D
        :param inter2: The second Interface1D object. 
        :type inter2: freestyle.types.Interface1D
        :rtype: bool 
        :return:  True or false. 
        '''
        pass


class ShapeUP1D:
    '''Class hierarchy: freestyle.types.UnaryPredicate1D > ShapeUP1D '''

    def __init__(self, first, second=0):
        '''Builds a ShapeUP1D object. 

        :param first: The first Id component. 
        :type first: int
        :param second: The second Id component. 
        :type second: int
        '''
        pass

    def __call__(self, inter):
        '''Returns true if the shape to which the Interface1D belongs to has the same freestyle.types.Id as the one specified by the user. 

        :param inter: An Interface1D object. 
        :type inter: freestyle.types.Interface1D
        :rtype: bool 
        :return:  True if Interface1D belongs to the shape of the user-specified Id. 
        '''
        pass


class TrueBP1D:
    '''Class hierarchy: freestyle.types.BinaryPredicate1D > TrueBP1D '''

    def __call__(self, inter1, inter2):
        '''Always returns true. 

        :param inter1: The first Interface1D object. 
        :type inter1: freestyle.types.Interface1D
        :param inter2: The second Interface1D object. 
        :type inter2: freestyle.types.Interface1D
        :rtype: bool 
        :return:  True. 
        '''
        pass


class TrueUP0D:
    '''Class hierarchy: freestyle.types.UnaryPredicate0D > TrueUP0D '''

    def __call__(self, it):
        '''Always returns true. 

        :param it: An Interface0DIterator object. 
        :type it: freestyle.types.Interface0DIterator
        :rtype: bool 
        :return:  True. 
        '''
        pass


class TrueUP1D:
    '''Class hierarchy: freestyle.types.UnaryPredicate1D > TrueUP1D '''

    def __call__(self, inter):
        '''Always returns true. 

        :param inter: An Interface1D object. 
        :type inter: freestyle.types.Interface1D
        :rtype: bool 
        :return:  True. 
        '''
        pass


class ViewMapGradientNormBP1D:
    '''Class hierarchy: freestyle.types.BinaryPredicate1D > ViewMapGradientNormBP1D '''

    def __init__(self,
                 level,
                 integration_type=IntegrationType.MEAN,
                 sampling=2.0):
        '''Builds a ViewMapGradientNormBP1D object. 

        :param level: The level of the pyramid from which the pixel must be read. 
        :type level: int
        :param integration_type: The integration method used to compute a single value from a set of values. 
        :type integration_type: freestyle.types.IntegrationType
        :param sampling: The resolution used to sample the chain: GetViewMapGradientNormF0D is evaluated at each sample point and the result is obtained by combining the resulting values into a single one, following the method specified by integration_type. 
        :type sampling: float
        '''
        pass

    def __call__(self, inter1, inter2):
        '''Returns true if the evaluation of the Gradient norm Function is higher for inter1 than for inter2. 

        :param inter1: The first Interface1D object. 
        :type inter1: freestyle.types.Interface1D
        :param inter2: The second Interface1D object. 
        :type inter2: freestyle.types.Interface1D
        :rtype: bool 
        :return:  True or false. 
        '''
        pass


class WithinImageBoundaryUP1D:
    '''Class hierarchy: freestyle.types.UnaryPredicate1D > WithinImageBoundaryUP1D '''

    def __init__(self, xmin, ymin, xmax, ymax):
        '''Builds an WithinImageBoundaryUP1D object. 

        :param xmin: X lower bound of the image boundary. 
        :type xmin: float
        :param ymin: Y lower bound of the image boundary. 
        :type ymin: float
        :param xmax: X upper bound of the image boundary. 
        :type xmax: float
        :param ymax: Y upper bound of the image boundary. 
        :type ymax: float
        '''
        pass

    def __call__(self, inter):
        '''Returns true if the Interface1D intersects with image boundary. 

        '''
        pass


class pyBackTVertexUP0D:
    '''Check whether an Interface0DIterator references a TVertex and is the one that is hidden (inferred from the context). '''

    pass


class pyClosedCurveUP1D:
    pass


class pyDensityFunctorUP1D:
    pass


class pyDensityUP1D:
    pass


class pyDensityVariableSigmaUP1D:
    pass


class pyHighDensityAnisotropyUP1D:
    pass


class pyHighDirectionalViewMapDensityUP1D:
    pass


class pyHighSteerableViewMapDensityUP1D:
    pass


class pyHighViewMapDensityUP1D:
    pass


class pyHighViewMapGradientNormUP1D:
    pass


class pyHigherCurvature2DAngleUP0D:
    pass


class pyHigherLengthUP1D:
    pass


class pyHigherNumberOfTurnsUP1D:
    pass


class pyIsInOccludersListUP1D:
    pass


class pyIsOccludedByIdListUP1D:
    pass


class pyIsOccludedByItselfUP1D:
    pass


class pyIsOccludedByUP1D:
    pass


class pyLengthBP1D:
    pass


class pyLowDirectionalViewMapDensityUP1D:
    pass


class pyLowSteerableViewMapDensityUP1D:
    pass


class pyNFirstUP1D:
    pass


class pyNatureBP1D:
    pass


class pyNatureUP1D:
    pass


class pyParameterUP0D:
    pass


class pyParameterUP0DGoodOne:
    pass


class pyProjectedXBP1D:
    pass


class pyProjectedYBP1D:
    pass


class pyShapeIdListUP1D:
    pass


class pyShapeIdUP1D:
    pass


class pyShuffleBP1D:
    pass


class pySilhouetteFirstBP1D:
    pass


class pyUEqualsUP0D:
    pass


class pyVertexNatureUP0D:
    pass


class pyViewMapGradientNormBP1D:
    pass


class pyZBP1D:
    pass


class pyZDiscontinuityBP1D:
    pass


class pyZSmallerUP1D:
    pass



CLIPPING = None
'''constant value 2 '''

KERNING_DEFAULT = None
'''constant value 8 '''

MONOCHROME = None
'''constant value 128 '''

ROTATION = None
'''constant value 1 '''

SHADOW = None
'''constant value 4 '''

WORD_WRAP = None
'''constant value 64 '''


def aspect(fontid, aspect):
    '''Set the aspect for drawing text. 

    :param fontid: The id of the typeface as returned by blf.load(), for default font use 0. 
    :type fontid: int
    :param aspect: The aspect ratio for text drawing to use. 
    :type aspect: float
    '''

    pass


def clipping(fontid, xmin, ymin, xmax, ymax):
    '''Set the clipping, enable/disable using CLIPPING. 

    :param fontid: The id of the typeface as returned by blf.load(), for default font use 0. 
    :type fontid: int
    :param xmin: Clip the drawing area by these bounds. 
    :type xmin: float
    :param ymin: Clip the drawing area by these bounds. 
    :type ymin: float
    :param xmax: Clip the drawing area by these bounds. 
    :type xmax: float
    :param ymax: Clip the drawing area by these bounds. 
    :type ymax: float
    '''

    pass


def color(fontid, r, g, b, a):
    '''Set the color for drawing text. 

    :param fontid: The id of the typeface as returned by blf.load(), for default font use 0. 
    :type fontid: int
    :param r: red channel 0.0 - 1.0. 
    :type r: float
    :param g: green channel 0.0 - 1.0. 
    :type g: float
    :param b: blue channel 0.0 - 1.0. 
    :type b: float
    :param a: alpha channel 0.0 - 1.0. 
    :type a: float
    '''

    pass


def dimensions(fontid, text):
    '''Return the width and height of the text. 

    :param fontid: The id of the typeface as returned by blf.load(), for default font use 0. 
    :type fontid: int
    :param text: the text to draw. 
    :type text: string
    :return:  the width and height of the text. 
    '''

    pass


def disable(fontid, option):
    '''Disable option. 

    :param fontid: The id of the typeface as returned by blf.load(), for default font use 0. 
    :type fontid: int
    :param option: One of ROTATION, CLIPPING, SHADOW or KERNING_DEFAULT. 
    :type option: int
    '''

    pass


def draw(fontid, text):
    '''Draw text in the current context. 

    :param fontid: The id of the typeface as returned by blf.load(), for default font use 0. 
    :type fontid: int
    :param text: the text to draw. 
    :type text: string
    '''

    pass


def enable(fontid, option):
    '''Enable option. 

    :param fontid: The id of the typeface as returned by blf.load(), for default font use 0. 
    :type fontid: int
    :param option: One of ROTATION, CLIPPING, SHADOW or KERNING_DEFAULT. 
    :type option: int
    '''

    pass


def load(filename):
    '''Load a new font. 

    :param filename: the filename of the font. 
    :type filename: string
    :return:  the new font’s fontid or -1 if there was an error. 
    '''

    pass


def position(fontid, x, y, z):
    '''Set the position for drawing text. 

    :param fontid: The id of the typeface as returned by blf.load(), for default font use 0. 
    :type fontid: int
    :param x: X axis position to draw the text. 
    :type x: float
    :param y: Y axis position to draw the text. 
    :type y: float
    :param z: Z axis position to draw the text. 
    :type z: float
    '''

    pass


def rotation(fontid, angle):
    '''Set the text rotation angle, enable/disable using ROTATION. 

    :param fontid: The id of the typeface as returned by blf.load(), for default font use 0. 
    :type fontid: int
    :param angle: The angle for text drawing to use. 
    :type angle: float
    '''

    pass


def shadow(fontid, level, r, g, b, a):
    '''Shadow options, enable/disable using SHADOW . 

    :param fontid: The id of the typeface as returned by blf.load(), for default font use 0. 
    :type fontid: int
    :param level: The blur level, can be 3, 5 or 0. 
    :type level: int
    :param r: Shadow color (red channel 0.0 - 1.0). 
    :type r: float
    :param g: Shadow color (green channel 0.0 - 1.0). 
    :type g: float
    :param b: Shadow color (blue channel 0.0 - 1.0). 
    :type b: float
    :param a: Shadow color (alpha channel 0.0 - 1.0). 
    :type a: float
    '''

    pass


def shadow_offset(fontid, x, y):
    '''Set the offset for shadow text. 

    :param fontid: The id of the typeface as returned by blf.load(), for default font use 0. 
    :type fontid: int
    :param x: Vertical shadow offset value in pixels. 
    :type x: float
    :param y: Horizontal shadow offset value in pixels. 
    :type y: float
    '''

    pass


def size(fontid, size, dpi):
    '''Set the size and dpi for drawing text. 

    :param fontid: The id of the typeface as returned by blf.load(), for default font use 0. 
    :type fontid: int
    :param size: Point size of the font. 
    :type size: int
    :param dpi: dots per inch value to use for drawing. 
    :type dpi: int
    '''

    pass


def unload(filename):
    '''Unload an existing font. 

    :param filename: the filename of the font. 
    :type filename: string
    '''

    pass


def word_wrap(fontid, wrap_width):
    '''Set the wrap width, enable/disable using WORD_WRAP. 

    :param fontid: The id of the typeface as returned by blf.load(), for default font use 0. 
    :type fontid: int
    :param wrap_width: The width (in pixels) to wrap words at. 
    :type wrap_width: int
    '''

    pass



class Buffer:
    '''The Buffer object is simply a block of memory that is delineated and initialized by the user. Many OpenGL functions return data to a C-style pointer, however, because this is not possible in python the Buffer object can be used to this end. Wherever pointer notation is used in the OpenGL functions the Buffer object can be used in it’s bgl wrapper. In some instances the Buffer object will need to be initialized with the template parameter, while in other instances the user will want to create just a blank buffer which will be zeroed by default. '''

    dimensions = None
    '''The number of dimensions of the Buffer. '''

    def to_list(self):
        '''The contents of the Buffer as a python list. 

        '''
        pass

    def __init__(self, type, dimensions, template=None):
        '''This will create a new Buffer object for use with other bgl OpenGL commands. Only the type of argument to store in the buffer and the dimensions of the buffer are necessary. Buffers are zeroed by default unless a template is supplied, in which case the buffer is initialized to the template. 

        :param type: The format to store data in. The type should be one of GL_BYTE, GL_SHORT, GL_INT, or GL_FLOAT. 
        :type type: int
        :param dimensions: If the dimensions are specified as an int a linear array will be created for the buffer. If a sequence is passed for the dimensions, the buffer becomes n-Dimensional, where n is equal to the number of parameters passed in the sequence. Example: [256,2] is a two- dimensional buffer while [256,256,4] creates a three- dimensional buffer. You can think of each additional dimension as a sub-item of the dimension to the left. i.e. [10,2] is a 10 element array each with 2 sub-items. [(0,0), (0,1), (1,0), (1,1), (2,0), …] etc. 
        :type dimensions: An int or sequence object specifying the dimensions of the buffer.
        :param template: A sequence of matching dimensions which will be used to initialize the Buffer. If a template is not passed in all fields will be initialized to 0. 
        :type template: A python sequence object (optional)
        :rtype: Buffer object 
        :return:  The newly created buffer as a PyObject. 
        '''
        pass


def glActiveTexture(texture):
    '''Select active texture unit. 

    :param texture: Constant in GL_TEXTURE0 0 - 8 
    :type texture: int
    '''

    pass


def glAttachShader(program, shader):
    '''Attaches a shader object to a program object. 

    :param program: Specifies the program object to which a shader object will be attached. 
    :type program: int
    :param shader: Specifies the shader object that is to be attached. 
    :type shader: int
    '''

    pass


def glBindTexture(target, texture):
    '''Bind a named texture to a texturing target 

    :param target: Specifies the target to which the texture is bound. 
    :type target: Enumerated constant
    :param texture: Specifies the name of a texture. 
    :type texture: unsigned int
    '''

    pass


def glBlendFunc(sfactor, dfactor):
    '''Specify pixel arithmetic 

    :param sfactor: Specifies how the red, green, blue, and alpha source blending factors are computed. 
    :type sfactor: Enumerated constant
    :param dfactor: Specifies how the red, green, blue, and alpha destination blending factors are computed. 
    :type dfactor: Enumerated constant
    '''

    pass


def glClear(mask):
    '''Clear buffers to preset values 

    :param mask: Bitwise OR of masks that indicate the buffers to be cleared. 
    :type mask: Enumerated constant(s)
    '''

    pass


def glClearColor(red, green, blue, alpha):
    '''Specify clear values for the color buffers 

    :param alpha: Specify the red, green, blue, and alpha values used when the color buffers are cleared. The initial values are all 0. 
    :type alpha: red,
    '''

    pass


def glClearDepth(depth):
    '''Specify the clear value for the depth buffer 

    :param depth: Specifies the depth value used when the depth buffer is cleared. The initial value is 1. 
    :type depth: int
    '''

    pass


def glClearStencil(s):
    '''Specify the clear value for the stencil buffer 

    :param s: Specifies the index used when the stencil buffer is cleared. The initial value is 0. 
    :type s: int
    '''

    pass


def glClipPlane(plane, equation):
    '''Specify a plane against which all geometry is clipped 

    :param plane: Specifies which clipping plane is being positioned. 
    :type plane: Enumerated constant
    :param equation: Specifies the address of an array of four double- precision floating-point values. These values are interpreted as a plane equation. 
    :type equation: bgl.Buffer object I{type GL_FLOAT}(double)
    '''

    pass


def glColor(red, green, blue, alpha):
    '''Set a new color. 

    :param blue: Specify new red, green, and blue values for the current color. 
    :type blue: red,
    :param alpha: Specifies a new alpha value for the current color. Included only in the four-argument glColor4 commands. (With ‘4’ colors only) 
    :type alpha: 
    '''

    pass


def glColorMask(red, green, blue, alpha):
    '''Enable and disable writing of frame buffer color components 

    :param alpha: Specify whether red, green, blue, and alpha can or cannot be written into the frame buffer. The initial values are all GL_TRUE, indicating that the color components can be written. 
    :type alpha: red,
    '''

    pass


def glCompileShader(shader):
    '''Compiles a shader object. 

    :param shader: Specifies the shader object to be compiled. 
    :type shader: int
    '''

    pass


def glCopyTexImage2D(target, level, internalformat, x, y, width, height,
                     border):
    '''Copy pixels into a 2D texture image 

    :param target: Specifies the target texture. 
    :type target: Enumerated constant
    :param level: Specifies the level-of-detail number. Level 0 is the base image level. Level n is the nth mipmap reduction image. 
    :type level: int
    :param internalformat: Specifies the number of color components in the texture. 
    :type internalformat: int
    :param y: Specify the window coordinates of the first pixel that is copied from the frame buffer. This location is the lower left corner of a rectangular block of pixels. 
    :type y: x,
    :param width: Specifies the width of the texture image. Must be 2n+2(border) for some integer n. All implementations support texture images that are at least 64 texels wide. 
    :type width: int
    :param height: Specifies the height of the texture image. Must be 2m+2(border) for some integer m. All implementations support texture images that are at least 64 texels high. 
    :type height: int
    :param border: Specifies the width of the border. Must be either 0 or 1. 
    :type border: int
    '''

    pass


def glCreateProgram():
    '''Creates a program object 

    :return:  The new program or zero if an error occurs. 
    '''

    pass


def glCreateShader(shaderType):
    '''Creates a shader object. 

    :param shaderType: 
    :type shaderType: Specifies the type of shader to be created. Must be one of GL_VERTEX_SHADER, GL_TESS_CONTROL_SHADER, GL_TESS_EVALUATION_SHADER, GL_GEOMETRY_SHADER, or GL_FRAGMENT_SHADER.
    :return:  0 if an error occurs. 
    '''

    pass


def glCullFace(mode):
    '''Specify whether front- or back-facing facets can be culled 

    :param mode: Specifies whether front- or back-facing facets are candidates for culling. 
    :type mode: Enumerated constant
    '''

    pass


def glDeleteProgram(program):
    '''Deletes a program object. 

    :param program: Specifies the program object to be deleted. 
    :type program: int
    '''

    pass


def glDeleteShader(shader):
    '''Deletes a shader object. 

    :param shader: Specifies the shader object to be deleted. 
    :type shader: int
    '''

    pass


def glDeleteTextures(n, textures):
    '''Delete named textures 

    :param n: Specifies the number of textures to be deleted 
    :type n: int
    :param textures: Specifies an array of textures to be deleted 
    :type textures: bgl.Buffer I{GL_INT}
    '''

    pass


def glDepthFunc(func):
    '''Specify the value used for depth buffer comparisons 

    :param func: Specifies the depth comparison function. 
    :type func: Enumerated constant
    '''

    pass


def glDepthMask(flag):
    '''Enable or disable writing into the depth buffer 

    :param flag: Specifies whether the depth buffer is enabled for writing. If flag is GL_FALSE, depth buffer writing is disabled. Otherwise, it is enabled. Initially, depth buffer writing is enabled. 
    :type flag: int (boolean)
    '''

    pass


def glDepthRange(zNear, zFar):
    '''Specify mapping of depth values from normalized device coordinates to window coordinates 

    :param zNear: Specifies the mapping of the near clipping plane to window coordinates. The initial value is 0. 
    :type zNear: int
    :param zFar: Specifies the mapping of the far clipping plane to window coordinates. The initial value is 1. 
    :type zFar: int
    '''

    pass


def glDetachShader(program, shader):
    '''Detaches a shader object from a program object to which it is attached. 

    :param program: Specifies the program object from which to detach the shader object. 
    :type program: int
    :param shader: pecifies the program object from which to detach the shader object. 
    :type shader: int
    '''

    pass


def glDisable(cap):
    '''Disable server-side GL capabilities 

    :param cap: Specifies a symbolic constant indicating a GL capability. 
    :type cap: Enumerated constant
    '''

    pass


def glDrawBuffer(mode):
    '''Specify which color buffers are to be drawn into 

    :param mode: Specifies up to four color buffers to be drawn into. 
    :type mode: Enumerated constant
    '''

    pass


def glEdgeFlag(flag):
    '''Flag edges as either boundary or non-boundary 

    :param flag: Specifies the current edge flag value.The initial value is GL_TRUE. 
    :type flag: Depends of function prototype
    '''

    pass


def glEnable(cap):
    '''Enable server-side GL capabilities 

    :param cap: Specifies a symbolic constant indicating a GL capability. 
    :type cap: Enumerated constant
    '''

    pass


def glEvalCoord(u, v):
    '''Evaluate enabled one- and two-dimensional maps 

    :param u: Specifies a value that is the domain coordinate u to the basis function defined in a previous glMap1 or glMap2 command. If the function prototype ends in ‘v’ then u specifies a pointer to an array containing either one or two domain coordinates. The first coordinate is u. The second coordinate is v, which is present only in glEvalCoord2 versions. 
    :type u: Depends on function prototype.
    :param v: Specifies a value that is the domain coordinate v to the basis function defined in a previous glMap2 command. This argument is not present in a glEvalCoord1 command. 
    :type v: Depends on function prototype. (only with '2' prototypes)
    '''

    pass


def glEvalMesh(mode, i1, i2):
    '''Compute a one- or two-dimensional grid of points or lines 

    :param mode: In glEvalMesh1, specifies whether to compute a one-dimensional mesh of points or lines. 
    :type mode: Enumerated constant
    :param i2: Specify the first and last integer values for the grid domain variable i. 
    :type i2: i1,
    '''

    pass


def glEvalPoint(i, j):
    '''Generate and evaluate a single point in a mesh 

    :param i: Specifies the integer value for grid domain variable i. 
    :type i: int
    :param j: Specifies the integer value for grid domain variable j (glEvalPoint2 only). 
    :type j: int (only with '2' prototypes)
    '''

    pass


def glFeedbackBuffer(size, type, buffer):
    '''Controls feedback mode 

    :param size: Specifies the maximum number of values that can be written into buffer. 
    :type size: int
    :param type: Specifies a symbolic constant that describes the information that will be returned for each vertex. 
    :type type: Enumerated constant
    :param buffer: Returns the feedback data. 
    :type buffer: bgl.Buffer object I{GL_FLOAT}
    '''

    pass


def glFinish():
    '''Block until all GL execution is complete 

    '''

    pass


def glFlush():
    '''Force Execution of GL commands in finite time 

    '''

    pass


def glFog(pname, param):
    '''Specify fog parameters 

    :param pname: Specifies a single-valued fog parameter. If the function prototype ends in ‘v’ specifies a fog parameter. 
    :type pname: Enumerated constant
    :param param: Specifies the value or values to be assigned to pname. GL_FOG_COLOR requires an array of four values. All other parameters accept an array containing only a single value. 
    :type param: Depends on function prototype.
    '''

    pass


def glFrontFace(mode):
    '''Define front- and back-facing polygons 

    :param mode: Specifies the orientation of front-facing polygons. 
    :type mode: Enumerated constant
    '''

    pass


def glGenTextures(n, textures):
    '''Generate texture names 

    :param n: Specifies the number of textures name to be generated. 
    :type n: int
    :param textures: Specifies an array in which the generated textures names are stored. 
    :type textures: bgl.Buffer object I{type GL_INT}
    '''

    pass


def glGet(pname, param):
    '''Return the value or values of a selected parameter 

    :param pname: Specifies the parameter value to be returned. 
    :type pname: Enumerated constant
    :param param: Returns the value or values of the specified parameter. 
    :type param: Depends on function prototype.
    '''

    pass


def glGetAttachedShaders(program, maxCount, count, shaders):
    '''Returns the handles of the shader objects attached to a program object. 

    :param program: Specifies the program object to be queried. 
    :type program: int
    :param maxCount: Specifies the size of the array for storing the returned object names. 
    :type maxCount: int
    :param count: Returns the number of names actually returned in objects. 
    :type count: bgl.Buffer int buffer.
    :param shaders: Specifies an array that is used to return the names of attached shader objects. 
    :type shaders: bgl.Buffer int buffer.
    '''

    pass


def glGetError():
    '''Return error information 

    '''

    pass


def glGetLight(light, pname, params):
    '''Return light source parameter values 

    :param light: Specifies a light source. The number of possible lights depends on the implementation, but at least eight lights are supported. They are identified by symbolic names of the form GL_LIGHTi where 0 < i < GL_MAX_LIGHTS. 
    :type light: Enumerated constant
    :param pname: Specifies a light source parameter for light. 
    :type pname: Enumerated constant
    :param params: Returns the requested data. 
    :type params: bgl.Buffer object. Depends on function prototype.
    '''

    pass


def glGetMap(target, query, v):
    '''Return evaluator parameters 

    :param target: Specifies the symbolic name of a map. 
    :type target: Enumerated constant
    :param query: Specifies which parameter to return. 
    :type query: Enumerated constant
    :param v: Returns the requested data. 
    :type v: bgl.Buffer object. Depends on function prototype.
    '''

    pass


def glGetMaterial(face, pname, params):
    '''Return material parameters 

    :param face: Specifies which of the two materials is being queried. representing the front and back materials, respectively. 
    :type face: Enumerated constant
    :param pname: Specifies the material parameter to return. 
    :type pname: Enumerated constant
    :param params: Returns the requested data. 
    :type params: bgl.Buffer object. Depends on function prototype.
    '''

    pass


def glGetPixelMap(map, values):
    '''Return the specified pixel map 

    :param map: Specifies the name of the pixel map to return. 
    :type map: Enumerated constant
    :param values: Returns the pixel map contents. 
    :type values: bgl.Buffer object. Depends on function prototype.
    '''

    pass


def glGetProgramInfoLog(program, maxLength, length, infoLog):
    '''Returns the information log for a program object. 

    :param program: Specifies the program object whose information log is to be queried. 
    :type program: int
    :param maxLength: Specifies the size of the character buffer for storing the returned information log. 
    :type maxLength: int
    :param length: Returns the length of the string returned in infoLog (excluding the null terminator). 
    :type length: bgl.Buffer int buffer.
    :param infoLog: Specifies an array of characters that is used to return the information log. 
    :type infoLog: bgl.Buffer char buffer.
    '''

    pass


def glGetProgramiv(program, pname, params):
    '''Returns a parameter from a program object. 

    :param program: Specifies the program object to be queried. 
    :type program: int
    :param pname: Specifies the object parameter. 
    :type pname: int
    :param params: Returns the requested object parameter. 
    :type params: bgl.Buffer int buffer.
    '''

    pass


def glGetShaderInfoLog(program, maxLength, length, infoLog):
    '''Returns the information log for a shader object. 

    :param shader: Specifies the shader object whose information log is to be queried. 
    :type shader: int
    :param maxLength: Specifies the size of the character buffer for storing the returned information log. 
    :type maxLength: int
    :param length: Returns the length of the string returned in infoLog (excluding the null terminator). 
    :type length: bgl.Buffer int buffer.
    :param infoLog: Specifies an array of characters that is used to return the information log. 
    :type infoLog: bgl.Buffer char buffer.
    '''

    pass


def glGetShaderSource(shader, bufSize, length, source):
    '''Returns the source code string from a shader object 

    :param shader: Specifies the shader object to be queried. 
    :type shader: int
    :param bufSize: Specifies the size of the character buffer for storing the returned source code string. 
    :type bufSize: int
    :param length: Returns the length of the string returned in source (excluding the null terminator). 
    :type length: bgl.Buffer int buffer.
    :param source: Specifies an array of characters that is used to return the source code string. 
    :type source: bgl.Buffer char.
    '''

    pass


def glGetString(name):
    '''Return a string describing the current GL connection 

    :param name: Specifies a symbolic constant. 
    :type name: Enumerated constant
    '''

    pass


def glGetTexEnv(target, pname, params):
    '''Return texture environment parameters 

    :param target: Specifies a texture environment. Must be GL_TEXTURE_ENV. 
    :type target: Enumerated constant
    :param pname: Specifies the symbolic name of a texture environment parameter. 
    :type pname: Enumerated constant
    :param params: Returns the requested data. 
    :type params: bgl.Buffer object. Depends on function prototype.
    '''

    pass


def glGetTexGen(coord, pname, params):
    '''Return texture coordinate generation parameters 

    :param coord: Specifies a texture coordinate. 
    :type coord: Enumerated constant
    :param pname: Specifies the symbolic name of the value(s) to be returned. 
    :type pname: Enumerated constant
    :param params: Returns the requested data. 
    :type params: bgl.Buffer object. Depends on function prototype.
    '''

    pass


def glGetTexImage(target, level, format, type, pixels):
    '''Return a texture image 

    :param target: Specifies which texture is to be obtained. 
    :type target: Enumerated constant
    :param level: Specifies the level-of-detail number of the desired image. Level 0 is the base image level. Level n is the nth mipmap reduction image. 
    :type level: int
    :param format: Specifies a pixel format for the returned data. 
    :type format: Enumerated constant
    :param type: Specifies a pixel type for the returned data. 
    :type type: Enumerated constant
    :param pixels: Returns the texture image. Should be a pointer to an array of the type specified by type 
    :type pixels: bgl.Buffer object.
    '''

    pass


def glGetTexLevelParameter(target, level, pname, params):
    '''return texture parameter values for a specific level of detail 

    :param target: Specifies the symbolic name of the target texture. 
    :type target: Enumerated constant
    :param level: Specifies the level-of-detail number of the desired image. Level 0 is the base image level. Level n is the nth mipmap reduction image. 
    :type level: int
    :param pname: Specifies the symbolic name of a texture parameter. 
    :type pname: Enumerated constant
    :param params: Returns the requested data. 
    :type params: bgl.Buffer object. Depends on function prototype.
    '''

    pass


def glGetTexParameter(target, pname, params):
    '''Return texture parameter values 

    :param target: Specifies the symbolic name of the target texture. 
    :type target: Enumerated constant
    :param pname: Specifies the symbolic name the target texture. 
    :type pname: Enumerated constant
    :param params: Returns the texture parameters. 
    :type params: bgl.Buffer object. Depends on function prototype.
    '''

    pass


def glHint(target, mode):
    '''Specify implementation-specific hints 

    :param target: Specifies a symbolic constant indicating the behavior to be controlled. 
    :type target: Enumerated constant
    :param mode: Specifies a symbolic constant indicating the desired behavior. 
    :type mode: Enumerated constant
    '''

    pass


def glIsEnabled(cap):
    '''Test whether a capability is enabled 

    :param cap: Specifies a constant representing a GL capability. 
    :type cap: Enumerated constant
    '''

    pass


def glIsProgram(program):
    '''Determines if a name corresponds to a program object 

    :param program: Specifies a potential program object. 
    :type program: int
    '''

    pass


def glIsShader(shader):
    '''Determines if a name corresponds to a shader object. 

    :param shader: Specifies a potential shader object. 
    :type shader: int
    '''

    pass


def glIsTexture(texture):
    '''Determine if a name corresponds to a texture 

    :param texture: Specifies a value that may be the name of a texture. 
    :type texture: unsigned int
    '''

    pass


def glLight(light, pname, param):
    '''Set the light source parameters 

    :param light: Specifies a light. The number of lights depends on the implementation, but at least eight lights are supported. They are identified by symbolic names of the form GL_LIGHTi where 0 < i < GL_MAX_LIGHTS. 
    :type light: Enumerated constant
    :param pname: Specifies a single-valued light source parameter for light. 
    :type pname: Enumerated constant
    :param param: Specifies the value that parameter pname of light source light will be set to. If function prototype ends in ‘v’ specifies a pointer to the value or values that parameter pname of light source light will be set to. 
    :type param: Depends on function prototype.
    '''

    pass


def glLightModel(pname, param):
    '''Set the lighting model parameters 

    :param pname: Specifies a single-value light model parameter. 
    :type pname: Enumerated constant
    :param param: Specifies the value that param will be set to. If function prototype ends in ‘v’ specifies a pointer to the value or values that param will be set to. 
    :type param: Depends on function prototype.
    '''

    pass


def glLineWidth(width):
    '''Specify the width of rasterized lines. 

    :param width: Specifies the width of rasterized lines. The initial value is 1. 
    :type width: float
    '''

    pass


def glLinkProgram(program):
    '''Links a program object. 

    :param program: Specifies the handle of the program object to be linked. 
    :type program: int
    '''

    pass


def glLoadMatrix(m):
    '''Replace the current matrix with the specified matrix 

    :param m: Specifies a pointer to 16 consecutive values, which are used as the elements of a 4x4 column-major matrix. 
    :type m: bgl.Buffer object. Depends on function prototype.
    '''

    pass


def glLogicOp(opcode):
    '''Specify a logical pixel operation for color index rendering 

    :param opcode: Specifies a symbolic constant that selects a logical operation. 
    :type opcode: Enumerated constant
    '''

    pass


def glMap1(target, u1, u2, stride, order, points):
    '''Define a one-dimensional evaluator 

    :param target: Specifies the kind of values that are generated by the evaluator. 
    :type target: Enumerated constant
    :param u2: Specify a linear mapping of u, as presented to glEvalCoord1, to ^, t he variable that is evaluated by the equations specified by this command. 
    :type u2: 
    :param stride: Specifies the number of floats or float (double)s between the beginning of one control point and the beginning of the next one in the data structure referenced in points. This allows control points to be embedded in arbitrary data structures. The only constraint is that the values for a particular control point must occupy contiguous memory locations. 
    :type stride: int
    :param order: Specifies the number of control points. Must be positive. 
    :type order: int
    :param points: Specifies a pointer to the array of control points. 
    :type points: bgl.Buffer object. Depends on function prototype.
    '''

    pass


def glMap2(target, u1, u2, ustride, uorder, v1, v2, vstride, vorder, points):
    '''Define a two-dimensional evaluator 

    :param target: Specifies the kind of values that are generated by the evaluator. 
    :type target: Enumerated constant
    :param u2: Specify a linear mapping of u, as presented to glEvalCoord2, to ^, t he variable that is evaluated by the equations specified by this command. Initially u1 is 0 and u2 is 1. 
    :type u2: 
    :param ustride: Specifies the number of floats or float (double)s between the beginning of control point R and the beginning of control point R ij, where i and j are the u and v control point indices, respectively. This allows control points to be embedded in arbitrary data structures. The only constraint is that the values for a particular control point must occupy contiguous memory locations. The initial value of ustride is 0. 
    :type ustride: int
    :param uorder: Specifies the dimension of the control point array in the u axis. Must be positive. The initial value is 1. 
    :type uorder: int
    :param v2: Specify a linear mapping of v, as presented to glEvalCoord2, to ^, one of the two variables that are evaluated by the equations specified by this command. Initially, v1 is 0 and v2 is 1. 
    :type v2: v1,
    :param vstride: Specifies the number of floats or float (double)s between the beginning of control point R and the beginning of control point R ij, where i and j are the u and v control point(indices, respectively. This allows control points to be embedded in arbitrary data structures. The only constraint is that the values for a particular control point must occupy contiguous memory locations. The initial value of vstride is 0. 
    :type vstride: int
    :param vorder: Specifies the dimension of the control point array in the v axis. Must be positive. The initial value is 1. 
    :type vorder: int
    :param points: Specifies a pointer to the array of control points. 
    :type points: bgl.Buffer object. Depends on function prototype.
    '''

    pass


def glMapGrid(un, u1, u2, vn, v1, v2):
    '''Define a one- or two-dimensional mesh 

    :param un: Specifies the number of partitions in the grid range interval [u1, u2]. Must be positive. 
    :type un: int
    :param u2: Specify the mappings for integer grid domain values i=0 and i=un. 
    :type u2: u1,
    :param vn: Specifies the number of partitions in the grid range interval [v1, v2] (glMapGrid2 only). 
    :type vn: int
    :param v2: Specify the mappings for integer grid domain values j=0 and j=vn (glMapGrid2 only). 
    :type v2: v1,
    '''

    pass


def glMaterial(face, pname, params):
    '''Specify material parameters for the lighting model. 

    :param face: Specifies which face or faces are being updated. Must be one of: 
    :type face: Enumerated constant
    :param pname: Specifies the single-valued material parameter of the face or faces that is being updated. Must be GL_SHININESS. 
    :type pname: Enumerated constant
    :param params: Specifies the value that parameter GL_SHININESS will be set to. If function prototype ends in ‘v’ specifies a pointer to the value or values that pname will be set to. 
    :type params: int
    '''

    pass


def glMultMatrix(m):
    '''Multiply the current matrix with the specified matrix 

    :param m: Points to 16 consecutive values that are used as the elements of a 4x4 column major matrix. 
    :type m: bgl.Buffer object. Depends on function prototype.
    '''

    pass


def glNormal3(nx, ny, nz, v):
    '''Set the current normal vector 

    :param nz: Specify the x, y, and z coordinates of the new current normal. The initial value of the current normal is the unit vector, (0, 0, 1). 
    :type nz: nx,
    :param v: Specifies a pointer to an array of three elements: the x, y, and z coordinates of the new current normal. 
    :type v: bgl.Buffer object. Depends on function prototype. (‘v’ prototypes)
    '''

    pass


def glPixelMap(map, mapsize, values):
    '''Set up pixel transfer maps 

    :param map: Specifies a symbolic map name. 
    :type map: Enumerated constant
    :param mapsize: Specifies the size of the map being defined. 
    :type mapsize: int
    :param values: Specifies an array of mapsize values. 
    :type values: bgl.Buffer object. Depends on function prototype.
    '''

    pass


def glPixelStore(pname, param):
    '''Set pixel storage modes 

    :param pname: Specifies the symbolic name of the parameter to be set. Six values affect the packing of pixel data into memory. Six more affect the unpacking of pixel data from memory. 
    :type pname: Enumerated constant
    :param param: Specifies the value that pname is set to. 
    :type param: Depends on function prototype.
    '''

    pass


def glPixelTransfer(pname, param):
    '''Set pixel transfer modes 

    :param pname: Specifies the symbolic name of the pixel transfer parameter to be set. 
    :type pname: Enumerated constant
    :param param: Specifies the value that pname is set to. 
    :type param: Depends on function prototype.
    '''

    pass


def glPointSize(size):
    '''Specify the diameter of rasterized points 

    :param size: Specifies the diameter of rasterized points. The initial value is 1. 
    :type size: float
    '''

    pass


def glPolygonMode(face, mode):
    '''Select a polygon rasterization mode 

    :param face: Specifies the polygons that mode applies to. Must be GL_FRONT for front-facing polygons, GL_BACK for back- facing polygons, or GL_FRONT_AND_BACK for front- and back-facing polygons. 
    :type face: Enumerated constant
    :param mode: Specifies how polygons will be rasterized. The initial value is GL_FILL for both front- and back- facing polygons. 
    :type mode: Enumerated constant
    '''

    pass


def glPolygonOffset(factor, units):
    '''Set the scale and units used to calculate depth values 

    :param factor: Specifies a scale factor that is used to create a variable depth offset for each polygon. The initial value is 0. 
    :type factor: float
    :param units: Is multiplied by an implementation-specific value to create a constant depth offset. The initial value is 0. 
    :type units: float
    '''

    pass


def glRasterPos(x, y, z, w):
    '''Specify the raster position for pixel operations 

    :param w: Specify the x,y,z, and w object coordinates (if present) for the raster position. If function prototype ends in ‘v’ specifies a pointer to an array of two, three, or four elements, specifying x, y, z, and w coordinates, respectively. 
    :type w: x,
    '''

    pass


def glReadBuffer(mode):
    '''Select a color buffer source for pixels. 

    :param mode: Specifies a color buffer. 
    :type mode: Enumerated constant
    '''

    pass


def glReadPixels(x, y, width, height, format, type, pixels):
    '''Read a block of pixels from the frame buffer 

    :param y: Specify the window coordinates of the first pixel that is read from the frame buffer. This location is the lower left corner of a rectangular block of pixels. 
    :type y: x,
    :param height: Specify the dimensions of the pixel rectangle. width and height of one correspond to a single pixel. 
    :type height: width,
    :param format: Specifies the format of the pixel data. 
    :type format: Enumerated constant
    :param type: Specifies the data type of the pixel data. 
    :type type: Enumerated constant
    :param pixels: Returns the pixel data. 
    :type pixels: bgl.Buffer object
    '''

    pass


def glRect(x1, y1, x2, y2, v1, v2):
    '''Draw a rectangle 

    :param y1: Specify one vertex of a rectangle 
    :type y1: x1,
    :param y2: Specify the opposite vertex of the rectangle 
    :type y2: x2,
    :param v2: Specifies a pointer to one vertex of a rectangle and the pointer to the opposite vertex of the rectangle 
    :type v2: v1,
    '''

    pass


def glRotate(angle, x, y, z):
    '''Multiply the current matrix by a rotation matrix 

    :param angle: Specifies the angle of rotation in degrees. 
    :type angle: Depends on function prototype.
    :param z: Specify the x, y, and z coordinates of a vector respectively. 
    :type z: x,
    '''

    pass


def glScale(x, y, z):
    '''Multiply the current matrix by a general scaling matrix 

    :param z: Specify scale factors along the x, y, and z axes, respectively. 
    :type z: x,
    '''

    pass


def glScissor(x, y, width, height):
    '''Define the scissor box 

    :param y: Specify the lower left corner of the scissor box. Initially (0, 0). 
    :type y: x,
    :param height: Specify the width and height of the scissor box. When a GL context is first attached to a window, width and height are set to the dimensions of that window. 
    :type height: width
    '''

    pass


def glShaderSource(shader, shader_string):
    '''Replaces the source code in a shader object. 

    :param shader: Specifies the handle of the shader object whose source code is to be replaced. 
    :type shader: int
    :param shader_string: The shader string. 
    :type shader_string: string
    '''

    pass


def glStencilFunc(func, ref, mask):
    '''Set function and reference value for stencil testing 

    :param func: Specifies the test function. 
    :type func: Enumerated constant
    :param ref: Specifies the reference value for the stencil test. ref is clamped to the range [0,2n-1], where n is the number of bitplanes in the stencil buffer. The initial value is 0. 
    :type ref: int
    :param mask: Specifies a mask that is ANDed with both the reference value and the stored stencil value when the test is done. The initial value is all 1’s. 
    :type mask: unsigned int
    '''

    pass


def glStencilMask(mask):
    '''Control the writing of individual bits in the stencil planes 

    :param mask: Specifies a bit mask to enable and disable writing of individual bits in the stencil planes. Initially, the mask is all 1’s. 
    :type mask: unsigned int
    '''

    pass


def glStencilOp(fail, zfail, zpass):
    '''Set stencil test actions 

    :param fail: Specifies the action to take when the stencil test fails. The initial value is GL_KEEP. 
    :type fail: Enumerated constant
    :param zfail: Specifies the stencil action when the stencil test passes, but the depth test fails. zfail accepts the same symbolic constants as fail. The initial value is GL_KEEP. 
    :type zfail: Enumerated constant
    :param zpass: Specifies the stencil action when both the stencil test and the depth test pass, or when the stencil test passes and either there is no depth buffer or depth testing is not enabled. zpass accepts the same symbolic constants as fail. The initial value is GL_KEEP. 
    :type zpass: Enumerated constant
    '''

    pass


def glTexCoord(s, t, r, q, v):
    '''Set the current texture coordinates 

    :param q: Specify s, t, r, and q texture coordinates. Not all parameters are present in all forms of the command. 
    :type q: s,
    :param v: Specifies a pointer to an array of one, two, three, or four elements, which in turn specify the s, t, r, and q texture coordinates. 
    :type v: bgl.Buffer object. Depends on function prototype. (for ‘v’ prototypes only)
    '''

    pass


def glTexEnv(target, pname, param):
    '''Set texture environment parameters 

    :param target: Specifies a texture environment. Must be GL_TEXTURE_ENV. 
    :type target: Enumerated constant
    :param pname: Specifies the symbolic name of a single-valued texture environment parameter. Must be GL_TEXTURE_ENV_MODE. 
    :type pname: Enumerated constant
    :param param: Specifies a single symbolic constant. If function prototype ends in ‘v’ specifies a pointer to a parameter array that contains either a single symbolic constant or an RGBA color 
    :type param: Depends on function prototype.
    '''

    pass


def glTexGen(coord, pname, param):
    '''Control the generation of texture coordinates 

    :param coord: Specifies a texture coordinate. 
    :type coord: Enumerated constant
    :param pname: Specifies the symbolic name of the texture- coordinate generation function. 
    :type pname: Enumerated constant
    :param param: Specifies a single-valued texture generation parameter. If function prototype ends in ‘v’ specifies a pointer to an array of texture generation parameters. If pname is GL_TEXTURE_GEN_MODE, then the array must contain a single symbolic constant. Otherwise, params holds the coefficients for the texture-coordinate generation function specified by pname. 
    :type param: Depends on function prototype.
    '''

    pass


def glTexImage1D(target, level, internalformat, width, border, format, type,
                 pixels):
    '''Specify a one-dimensional texture image 

    :param target: Specifies the target texture. 
    :type target: Enumerated constant
    :param level: Specifies the level-of-detail number. Level 0 is the base image level. Level n is the nth mipmap reduction image. 
    :type level: int
    :param internalformat: Specifies the number of color components in the texture. 
    :type internalformat: int
    :param width: Specifies the width of the texture image. Must be 2n+2(border) for some integer n. All implementations support texture images that are at least 64 texels wide. The height of the 1D texture image is 1. 
    :type width: int
    :param border: Specifies the width of the border. Must be either 0 or 1. 
    :type border: int
    :param format: Specifies the format of the pixel data. 
    :type format: Enumerated constant
    :param type: Specifies the data type of the pixel data. 
    :type type: Enumerated constant
    :param pixels: Specifies a pointer to the image data in memory. 
    :type pixels: bgl.Buffer object.
    '''

    pass


def glTexImage2D(target, level, internalformat, width, height, border, format,
                 type, pixels):
    '''Specify a two-dimensional texture image 

    :param target: Specifies the target texture. 
    :type target: Enumerated constant
    :param level: Specifies the level-of-detail number. Level 0 is the base image level. Level n is the nth mipmap reduction image. 
    :type level: int
    :param internalformat: Specifies the number of color components in the texture. 
    :type internalformat: int
    :param width: Specifies the width of the texture image. Must be 2n+2(border) for some integer n. All implementations support texture images that are at least 64 texels wide. 
    :type width: int
    :param height: Specifies the height of the texture image. Must be 2m+2(border) for some integer m. All implementations support texture images that are at least 64 texels high. 
    :type height: int
    :param border: Specifies the width of the border. Must be either 0 or 1. 
    :type border: int
    :param format: Specifies the format of the pixel data. 
    :type format: Enumerated constant
    :param type: Specifies the data type of the pixel data. 
    :type type: Enumerated constant
    :param pixels: Specifies a pointer to the image data in memory. 
    :type pixels: bgl.Buffer object.
    '''

    pass


def glTexParameter(target, pname, param):
    '''Set texture parameters 

    :param target: Specifies the target texture. 
    :type target: Enumerated constant
    :param pname: Specifies the symbolic name of a single-valued texture parameter. 
    :type pname: Enumerated constant
    :param param: Specifies the value of pname. If function prototype ends in ‘v’ specifies a pointer to an array where the value or values of pname are stored. 
    :type param: Depends on function prototype.
    '''

    pass


def glTranslate(x, y, z):
    '''Multiply the current matrix by a translation matrix 

    :param z: Specify the x, y, and z coordinates of a translation vector. 
    :type z: x,
    '''

    pass


def glUseProgram(program):
    '''Installs a program object as part of current rendering state 

    :param program: Specifies the handle of the program object whose executables are to be used as part of current rendering state. 
    :type program: int
    '''

    pass


def glValidateProgram(program):
    '''Validates a program object 

    :param program: Specifies the handle of the program object to be validated. 
    :type program: int
    '''

    pass


def glVertex(x, y, z, w, v):
    '''Specify a vertex 

    :param w: Specify x, y, z, and w coordinates of a vertex. Not all parameters are present in all forms of the command. 
    :type w: x,
    :param v: Specifies a pointer to an array of two, three, or four elements. The elements of a two-element array are x and y; of a three-element array, x, y, and z; and of a four-element array, x, y, z, and w. 
    :type v: bgl.Buffer object. Depends of function prototype (for ‘v’ prototypes only)
    '''

    pass


def glViewport(x, y, width, height):
    '''Set the viewport 

    :param y: Specify the lower left corner of the viewport rectangle, in pixels. The initial value is (0,0). 
    :type y: x,
    :param height: Specify the width and height of the viewport. When a GL context is first attached to a window, width and height are set to the dimensions of that window. 
    :type height: width,
    '''

    pass



STDPERLIN = None

STDPERLIN = None

STDPERLIN = None

STDPERLIN = None

STDPERLIN = None

STDPERLIN = None


def cell(position):
    '''Returns cell noise value at the specified position. 

    :param position: The position to evaluate the selected noise function. 
    :type position: mathutils.Vector
    :return:  The cell noise value. 
    '''

    pass


def cell_vector(position):
    '''Returns cell noise vector at the specified position. 

    :param position: The position to evaluate the selected noise function. 
    :type position: mathutils.Vector
    :return:  The cell noise vector. 
    '''

    pass


def fractal(position, H, lacunarity, octaves, noise_basis='PERLIN_ORIGINAL'):
    '''Returns the fractal Brownian motion (fBm) noise value from the noise basis at the specified position. 

    :param position: The position to evaluate the selected noise function. 
    :type position: mathutils.Vector
    :param H: The fractal increment factor. 
    :type H: float
    :param lacunarity: The gap between successive frequencies. 
    :type lacunarity: float
    :param octaves: The number of different noise frequencies used. 
    :type octaves: int
    :param noise_basis: Enumerator in [‘BLENDER’, ‘PERLIN_ORIGINAL’, ‘PERLIN_NEW’, ‘VORONOI_F1’, ‘VORONOI_F2’, ‘VORONOI_F3’, ‘VORONOI_F4’, ‘VORONOI_F2F1’, ‘VORONOI_CRACKLE’, ‘CELLNOISE’]. 
    :type noise_basis: string
    :return:  The fractal Brownian motion noise value. 
    '''

    pass


def hetero_terrain(position,
                   H,
                   lacunarity,
                   octaves,
                   offset,
                   noise_basis='PERLIN_ORIGINAL'):
    '''Returns the heterogeneous terrain value from the noise basis at the specified position. 

    :param position: The position to evaluate the selected noise function. 
    :type position: mathutils.Vector
    :param H: The fractal dimension of the roughest areas. 
    :type H: float
    :param lacunarity: The gap between successive frequencies. 
    :type lacunarity: float
    :param octaves: The number of different noise frequencies used. 
    :type octaves: int
    :param offset: The height of the terrain above ‘sea level’. 
    :type offset: float
    :param noise_basis: Enumerator in [‘BLENDER’, ‘PERLIN_ORIGINAL’, ‘PERLIN_NEW’, ‘VORONOI_F1’, ‘VORONOI_F2’, ‘VORONOI_F3’, ‘VORONOI_F4’, ‘VORONOI_F2F1’, ‘VORONOI_CRACKLE’, ‘CELLNOISE’]. 
    :type noise_basis: string
    :return:  The heterogeneous terrain value. 
    '''

    pass


def hybrid_multi_fractal(position,
                         H,
                         lacunarity,
                         octaves,
                         offset,
                         gain,
                         noise_basis='PERLIN_ORIGINAL'):
    '''Returns hybrid multifractal value from the noise basis at the specified position. 

    :param position: The position to evaluate the selected noise function. 
    :type position: mathutils.Vector
    :param H: The fractal dimension of the roughest areas. 
    :type H: float
    :param lacunarity: The gap between successive frequencies. 
    :type lacunarity: float
    :param octaves: The number of different noise frequencies used. 
    :type octaves: int
    :param offset: The height of the terrain above ‘sea level’. 
    :type offset: float
    :param gain: Scaling applied to the values. 
    :type gain: float
    :param noise_basis: Enumerator in [‘BLENDER’, ‘PERLIN_ORIGINAL’, ‘PERLIN_NEW’, ‘VORONOI_F1’, ‘VORONOI_F2’, ‘VORONOI_F3’, ‘VORONOI_F4’, ‘VORONOI_F2F1’, ‘VORONOI_CRACKLE’, ‘CELLNOISE’]. 
    :type noise_basis: string
    :return:  The hybrid multifractal value. 
    '''

    pass


def multi_fractal(position,
                  H,
                  lacunarity,
                  octaves,
                  noise_basis='PERLIN_ORIGINAL'):
    '''Returns multifractal noise value from the noise basis at the specified position. 

    :param position: The position to evaluate the selected noise function. 
    :type position: mathutils.Vector
    :param H: The fractal increment factor. 
    :type H: float
    :param lacunarity: The gap between successive frequencies. 
    :type lacunarity: float
    :param octaves: The number of different noise frequencies used. 
    :type octaves: int
    :param noise_basis: Enumerator in [‘BLENDER’, ‘PERLIN_ORIGINAL’, ‘PERLIN_NEW’, ‘VORONOI_F1’, ‘VORONOI_F2’, ‘VORONOI_F3’, ‘VORONOI_F4’, ‘VORONOI_F2F1’, ‘VORONOI_CRACKLE’, ‘CELLNOISE’]. 
    :type noise_basis: string
    :return:  The multifractal noise value. 
    '''

    pass


def noise(position, noise_basis='PERLIN_ORIGINAL'):
    '''Returns noise value from the noise basis at the position specified. 

    :param position: The position to evaluate the selected noise function. 
    :type position: mathutils.Vector
    :param noise_basis: Enumerator in [‘BLENDER’, ‘PERLIN_ORIGINAL’, ‘PERLIN_NEW’, ‘VORONOI_F1’, ‘VORONOI_F2’, ‘VORONOI_F3’, ‘VORONOI_F4’, ‘VORONOI_F2F1’, ‘VORONOI_CRACKLE’, ‘CELLNOISE’]. 
    :type noise_basis: string
    :return:  The noise value. 
    '''

    pass


def noise_vector(position, noise_basis='PERLIN_ORIGINAL'):
    '''Returns the noise vector from the noise basis at the specified position. 

    :param position: The position to evaluate the selected noise function. 
    :type position: mathutils.Vector
    :param noise_basis: Enumerator in [‘BLENDER’, ‘PERLIN_ORIGINAL’, ‘PERLIN_NEW’, ‘VORONOI_F1’, ‘VORONOI_F2’, ‘VORONOI_F3’, ‘VORONOI_F4’, ‘VORONOI_F2F1’, ‘VORONOI_CRACKLE’, ‘CELLNOISE’]. 
    :type noise_basis: string
    :return:  The noise vector. 
    '''

    pass


def random():
    '''Returns a random number in the range [0, 1). 

    :return:  The random number. 
    '''

    pass


def random_unit_vector(size=3):
    '''Returns a unit vector with random entries. 

    :param size: The size of the vector to be produced, in the range [2, 4]. 
    :type size: int
    :return:  The random unit vector. 
    '''

    pass


def random_vector(size=3):
    '''Returns a vector with random entries in the range (-1, 1). 

    :param size: The size of the vector to be produced. 
    :type size: int
    :return:  The random vector. 
    '''

    pass


def ridged_multi_fractal(position,
                         H,
                         lacunarity,
                         octaves,
                         offset,
                         gain,
                         noise_basis='PERLIN_ORIGINAL'):
    '''Returns ridged multifractal value from the noise basis at the specified position. 

    :param position: The position to evaluate the selected noise function. 
    :type position: mathutils.Vector
    :param H: The fractal dimension of the roughest areas. 
    :type H: float
    :param lacunarity: The gap between successive frequencies. 
    :type lacunarity: float
    :param octaves: The number of different noise frequencies used. 
    :type octaves: int
    :param offset: The height of the terrain above ‘sea level’. 
    :type offset: float
    :param gain: Scaling applied to the values. 
    :type gain: float
    :param noise_basis: Enumerator in [‘BLENDER’, ‘PERLIN_ORIGINAL’, ‘PERLIN_NEW’, ‘VORONOI_F1’, ‘VORONOI_F2’, ‘VORONOI_F3’, ‘VORONOI_F4’, ‘VORONOI_F2F1’, ‘VORONOI_CRACKLE’, ‘CELLNOISE’]. 
    :type noise_basis: string
    :return:  The ridged multifractal value. 
    '''

    pass


def seed_set(seed):
    '''Sets the random seed used for random_unit_vector, and random. 

    :param seed: Seed used for the random generator. When seed is zero, the current time will be used instead. 
    :type seed: int
    '''

    pass


def turbulence(position,
               octaves,
               hard,
               noise_basis='PERLIN_ORIGINAL',
               amplitude_scale=0.5,
               frequency_scale=2.0):
    '''Returns the turbulence value from the noise basis at the specified position. 

    :param position: The position to evaluate the selected noise function. 
    :type position: mathutils.Vector
    :param octaves: The number of different noise frequencies used. 
    :type octaves: int
    :param hard: Specifies whether returned turbulence is hard (sharp transitions) or soft (smooth transitions). 
    :type hard: boolean
    :param noise_basis: Enumerator in [‘BLENDER’, ‘PERLIN_ORIGINAL’, ‘PERLIN_NEW’, ‘VORONOI_F1’, ‘VORONOI_F2’, ‘VORONOI_F3’, ‘VORONOI_F4’, ‘VORONOI_F2F1’, ‘VORONOI_CRACKLE’, ‘CELLNOISE’]. 
    :type noise_basis: string
    :param amplitude_scale: The amplitude scaling factor. 
    :type amplitude_scale: float
    :param frequency_scale: The frequency scaling factor 
    :type frequency_scale: float
    :return:  The turbulence value. 
    '''

    pass


def turbulence_vector(position,
                      octaves,
                      hard,
                      noise_basis='PERLIN_ORIGINAL',
                      amplitude_scale=0.5,
                      frequency_scale=2.0):
    '''Returns the turbulence vector from the noise basis at the specified position. 

    :param position: The position to evaluate the selected noise function. 
    :type position: mathutils.Vector
    :param octaves: The number of different noise frequencies used. 
    :type octaves: int
    :param hard: Specifies whether returned turbulence is hard (sharp transitions) or soft (smooth transitions). 
    :type hard: :boolean
    :param noise_basis: Enumerator in [‘BLENDER’, ‘PERLIN_ORIGINAL’, ‘PERLIN_NEW’, ‘VORONOI_F1’, ‘VORONOI_F2’, ‘VORONOI_F3’, ‘VORONOI_F4’, ‘VORONOI_F2F1’, ‘VORONOI_CRACKLE’, ‘CELLNOISE’]. 
    :type noise_basis: string
    :param amplitude_scale: The amplitude scaling factor. 
    :type amplitude_scale: float
    :param frequency_scale: The frequency scaling factor 
    :type frequency_scale: float
    :return:  The turbulence vector. 
    '''

    pass


def variable_lacunarity(position,
                        distortion,
                        noise_type1='PERLIN_ORIGINAL',
                        noise_type2='PERLIN_ORIGINAL'):
    '''Returns variable lacunarity noise value, a distorted variety of noise, from noise type 1 distorted by noise type 2 at the specified position. 

    :param position: The position to evaluate the selected noise function. 
    :type position: mathutils.Vector
    :param distortion: The amount of distortion. 
    :type distortion: float
    :param noise_type1: Enumerator in [‘BLENDER’, ‘PERLIN_ORIGINAL’, ‘PERLIN_NEW’, ‘VORONOI_F1’, ‘VORONOI_F2’, ‘VORONOI_F3’, ‘VORONOI_F4’, ‘VORONOI_F2F1’, ‘VORONOI_CRACKLE’, ‘CELLNOISE’]. 
    :type noise_type1: string
    :param noise_type2: Enumerator in [‘BLENDER’, ‘PERLIN_ORIGINAL’, ‘PERLIN_NEW’, ‘VORONOI_F1’, ‘VORONOI_F2’, ‘VORONOI_F3’, ‘VORONOI_F4’, ‘VORONOI_F2F1’, ‘VORONOI_CRACKLE’, ‘CELLNOISE’]. 
    :type noise_type2: string
    :return:  The variable lacunarity noise value. 
    '''

    pass


def voronoi(position, distance_metric='DISTANCE', exponent=2.5):
    '''Returns a list of distances to the four closest features and their locations. 

    :param position: The position to evaluate the selected noise function. 
    :type position: mathutils.Vector
    :param distance_metric: Enumerator in [‘DISTANCE’, ‘DISTANCE_SQUARED’, ‘MANHATTAN’, ‘CHEBYCHEV’, ‘MINKOVSKY’, ‘MINKOVSKY_HALF’, ‘MINKOVSKY_FOUR’]. 
    :type distance_metric: string
    :param exponent: The exponent for Minkowski distance metric. 
    :type exponent: float
    :return:  A list of distances to the four closest features and their locations. 
    '''

    pass



class BVHTree:
    def find_nearest(self, origin, distance=1.84467e+19):
        '''Find the nearest element (typically face index) to a point. 

        :param co: Find nearest element to this point. 
        :type co: Vector
        :param distance: Maximum distance threshold. 
        :type distance: float
        :rtype: tuple 
        :return:  Returns a tuple (Vector location, Vector normal, int index, float distance), Values will all be None if no hit is found. 
        '''
        pass

    def find_nearest_range(self, origin, distance=1.84467e+19):
        '''Find the nearest elements (typically face index) to a point in the distance range. 

        :param co: Find nearest elements to this point. 
        :type co: Vector
        :param distance: Maximum distance threshold. 
        :type distance: float
        :rtype: list 
        :return:  Returns a list of tuples (Vector location, Vector normal, int index, float distance), 
        '''
        pass

    def overlap(self, other_tree):
        '''Find overlapping indices between 2 trees. 

        :param other_tree: Other tree to perform overlap test on. 
        :type other_tree: BVHTree
        :rtype: list 
        :return:  Returns a list of unique index pairs, the first index referencing this tree, the second referencing the other_tree. 
        '''
        pass

    def ray_cast(self, origin, direction, distance=sys.float_info.max):
        '''Cast a ray onto the mesh. 

        :param co: Start location of the ray in object space. 
        :type co: Vector
        :param direction: Direction of the ray in object space. 
        :type direction: Vector
        :param distance: Maximum distance threshold. 
        :type distance: float
        :rtype: tuple 
        :return:  Returns a tuple (Vector location, Vector normal, int index, float distance), Values will all be None if no hit is found. 
        '''
        pass



from .context import Context as context

from . import types
from . import ops
from . import app
from . import utils
from . import context
from . import path
from . import props

data = None  # type:  bpy.types.BlendData
'''Access to Blenders internal data '''



def area_tri(v1, v2, v3):
    '''Returns the area size of the 2D or 3D triangle defined. 

    :param v1: Point1 
    :type v1: mathutils.Vector
    :param v2: Point2 
    :type v2: mathutils.Vector
    :param v3: Point3 
    :type v3: mathutils.Vector
    '''

    pass


def barycentric_transform(point, tri_a1, tri_a2, tri_a3, tri_b1, tri_b2,
                          tri_b3):
    '''Return a transformed point, the transformation is defined by 2 triangles. 

    :param point: The point to transform. 
    :type point: mathutils.Vector
    :param tri_a1: source triangle vertex. 
    :type tri_a1: mathutils.Vector
    :param tri_a2: source triangle vertex. 
    :type tri_a2: mathutils.Vector
    :param tri_a3: source triangle vertex. 
    :type tri_a3: mathutils.Vector
    :param tri_b1: target triangle vertex. 
    :type tri_b1: mathutils.Vector
    :param tri_b2: target triangle vertex. 
    :type tri_b2: mathutils.Vector
    :param tri_b3: target triangle vertex. 
    :type tri_b3: mathutils.Vector
    :return:  The transformed point 
    '''

    pass


def box_fit_2d(points):
    '''Returns an angle that best fits the points to an axis aligned rectangle 

    :param points: list of 2d points. 
    :type points: list
    :return:  angle 
    '''

    pass


def box_pack_2d(boxes):
    '''Returns the normal of the 3D tri or quad. 

    :param boxes: list of boxes, each box is a list where the first 4 items are [x, y, width, height, …] other items are ignored. 
    :type boxes: list
    :return:  the width and height of the packed bounding box 
    '''

    pass


def convex_hull_2d(points):
    '''Returns a list of indices into the list given 

    :param points: list of 2d points. 
    :type points: list
    :return:  a list of indices 
    '''

    pass


def distance_point_to_plane(pt, plane_co, plane_no):
    '''Returns the signed distance between a point and a plane (negative when below the normal). 

    :param pt: Point 
    :type pt: mathutils.Vector
    :param plane_co: A point on the plane 
    :type plane_co: mathutils.Vector
    :param plane_no: The direction the plane is facing 
    :type plane_no: mathutils.Vector
    '''

    pass


def interpolate_bezier(knot1, handle1, handle2, knot2, resolution):
    '''Interpolate a bezier spline segment. 

    :param knot1: First bezier spline point. 
    :type knot1: mathutils.Vector
    :param handle1: First bezier spline handle. 
    :type handle1: mathutils.Vector
    :param handle2: Second bezier spline handle. 
    :type handle2: mathutils.Vector
    :param knot2: Second bezier spline point. 
    :type knot2: mathutils.Vector
    :param resolution: Number of points to return. 
    :type resolution: int
    :return:  The interpolated points 
    '''

    pass


def intersect_line_line(v1, v2, v3, v4):
    '''Returns a tuple with the points on each line respectively closest to the other. 

    :param v1: First point of the first line 
    :type v1: mathutils.Vector
    :param v2: Second point of the first line 
    :type v2: mathutils.Vector
    :param v3: First point of the second line 
    :type v3: mathutils.Vector
    :param v4: Second point of the second line 
    :type v4: mathutils.Vector
    '''

    pass


def intersect_line_line_2d(lineA_p1, lineA_p2, lineB_p1, lineB_p2):
    '''Takes 2 segments (defined by 4 vectors) and returns a vector for their point of intersection or None. 

    :param lineA_p1: First point of the first line 
    :type lineA_p1: mathutils.Vector
    :param lineA_p2: Second point of the first line 
    :type lineA_p2: mathutils.Vector
    :param lineB_p1: First point of the second line 
    :type lineB_p1: mathutils.Vector
    :param lineB_p2: Second point of the second line 
    :type lineB_p2: mathutils.Vector
    :return:  The point of intersection or None when not found 
    '''

    pass


def intersect_line_plane(line_a, line_b, plane_co, plane_no, no_flip=False):
    '''Calculate the intersection between a line (as 2 vectors) and a plane. Returns a vector for the intersection or None. 

    :param line_a: First point of the first line 
    :type line_a: mathutils.Vector
    :param line_b: Second point of the first line 
    :type line_b: mathutils.Vector
    :param plane_co: A point on the plane 
    :type plane_co: mathutils.Vector
    :param plane_no: The direction the plane is facing 
    :type plane_no: mathutils.Vector
    :return:  The point of intersection or None when not found 
    '''

    pass


def intersect_line_sphere(line_a, line_b, sphere_co, sphere_radius, clip=True):
    '''Takes a line (as 2 points) and a sphere (as a point and a radius) and returns the intersection 

    :param line_a: First point of the line 
    :type line_a: mathutils.Vector
    :param line_b: Second point of the line 
    :type line_b: mathutils.Vector
    :param sphere_co: The center of the sphere 
    :type sphere_co: mathutils.Vector
    :param sphere_radius: Radius of the sphere 
    :type sphere_radius: sphere_radius
    :return:  The intersection points as a pair of vectors or None when there is no intersection 
    '''

    pass


def intersect_line_sphere_2d(line_a,
                             line_b,
                             sphere_co,
                             sphere_radius,
                             clip=True):
    '''Takes a line (as 2 points) and a sphere (as a point and a radius) and returns the intersection 

    :param line_a: First point of the line 
    :type line_a: mathutils.Vector
    :param line_b: Second point of the line 
    :type line_b: mathutils.Vector
    :param sphere_co: The center of the sphere 
    :type sphere_co: mathutils.Vector
    :param sphere_radius: Radius of the sphere 
    :type sphere_radius: sphere_radius
    :return:  The intersection points as a pair of vectors or None when there is no intersection 
    '''

    pass


def intersect_plane_plane(plane_a_co, plane_a_no, plane_b_co, plane_b_no):
    '''Return the intersection between two planes 

    :param plane_a_co: Point on the first plane 
    :type plane_a_co: mathutils.Vector
    :param plane_a_no: Normal of the first plane 
    :type plane_a_no: mathutils.Vector
    :param plane_b_co: Point on the second plane 
    :type plane_b_co: mathutils.Vector
    :param plane_b_no: Normal of the second plane 
    :type plane_b_no: mathutils.Vector
    :return:  The line of the intersection represented as a point and a vector 
    '''

    pass


def intersect_point_line(pt, line_p1, line_p2):
    '''Takes a point and a line and returns a tuple with the closest point on the line and its distance from the first point of the line as a percentage of the length of the line. 

    :param pt: Point 
    :type pt: mathutils.Vector
    :param line_p1: First point of the line 
    :type line_p1: mathutils.Vector
    :param line_p1: Second point of the line 
    :type line_p1: 
    '''

    pass


def intersect_point_quad_2d(pt, quad_p1, quad_p2, quad_p3, quad_p4):
    '''Takes 5 vectors (using only the x and y coordinates): one is the point and the next 4 define the quad, only the x and y are used from the vectors. Returns 1 if the point is within the quad, otherwise 0. Works only with convex quads without singular edges. 

    :param pt: Point 
    :type pt: mathutils.Vector
    :param quad_p1: First point of the quad 
    :type quad_p1: mathutils.Vector
    :param quad_p2: Second point of the quad 
    :type quad_p2: mathutils.Vector
    :param quad_p3: Third point of the quad 
    :type quad_p3: mathutils.Vector
    :param quad_p4: Fourth point of the quad 
    :type quad_p4: mathutils.Vector
    '''

    pass


def intersect_point_tri(pt, tri_p1, tri_p2, tri_p3):
    '''Takes 4 vectors: one is the point and the next 3 define the triangle. 

    :param pt: Point 
    :type pt: mathutils.Vector
    :param tri_p1: First point of the triangle 
    :type tri_p1: mathutils.Vector
    :param tri_p2: Second point of the triangle 
    :type tri_p2: mathutils.Vector
    :param tri_p3: Third point of the triangle 
    :type tri_p3: mathutils.Vector
    :return:  Point on the triangles plane or None if its outside the triangle 
    '''

    pass


def intersect_point_tri_2d(pt, tri_p1, tri_p2, tri_p3):
    '''Takes 4 vectors (using only the x and y coordinates): one is the point and the next 3 define the triangle. Returns 1 if the point is within the triangle, otherwise 0. 

    :param pt: Point 
    :type pt: mathutils.Vector
    :param tri_p1: First point of the triangle 
    :type tri_p1: mathutils.Vector
    :param tri_p2: Second point of the triangle 
    :type tri_p2: mathutils.Vector
    :param tri_p3: Third point of the triangle 
    :type tri_p3: mathutils.Vector
    '''

    pass


def intersect_ray_tri(v1, v2, v3, ray, orig, clip=True):
    '''Returns the intersection between a ray and a triangle, if possible, returns None otherwise. 

    :param v1: Point1 
    :type v1: mathutils.Vector
    :param v2: Point2 
    :type v2: mathutils.Vector
    :param v3: Point3 
    :type v3: mathutils.Vector
    :param ray: Direction of the projection 
    :type ray: mathutils.Vector
    :param orig: Origin 
    :type orig: mathutils.Vector
    :param clip: When False, don’t restrict the intersection to the area of the triangle, use the infinite plane defined by the triangle. 
    :type clip: boolean
    :return:  The point of intersection or None if no intersection is found 
    '''

    pass


def intersect_sphere_sphere_2d(p_a, radius_a, p_b, radius_b):
    '''Returns 2 points on between intersecting circles. 

    :param p_a: Center of the first circle 
    :type p_a: mathutils.Vector
    :param radius_a: Radius of the first circle 
    :type radius_a: float
    :param p_b: Center of the second circle 
    :type p_b: mathutils.Vector
    :param radius_b: Radius of the second circle 
    :type radius_b: float
    '''

    pass


def normal(vectors):
    '''Returns the normal of a 3D polygon. 

    :param vectors: Vectors to calculate normals with 
    :type vectors: sequence of 3 or more 3d vector
    '''

    pass


def points_in_planes(planes):
    '''Returns a list of points inside all planes given and a list of index values for the planes used. 

    :param planes: List of planes (4D vectors). 
    :type planes: list of mathutils.Vector
    :return:  two lists, once containing the vertices inside the planes, another containing the plane indices used 
    '''

    pass


def tessellate_polygon(veclist_list):
    '''Takes a list of polylines (each point a vector) and returns the point indices for a polyline filled with triangles. 

    :param veclist_list: list of polylines 
    :type veclist_list: 
    '''

    pass


def volume_tetrahedron(v1, v2, v3, v4):
    '''Return the volume formed by a tetrahedron (points can be in any order). 

    :param v1: Point1 
    :type v1: mathutils.Vector
    :param v2: Point2 
    :type v2: mathutils.Vector
    :param v3: Point3 
    :type v3: mathutils.Vector
    :param v4: Point4 
    :type v4: mathutils.Vector
    '''

    pass



def intersect_face_point(face, point):
    '''Tests if the projection of a point is inside a face (using the face’s normal). 

    :param face: The face to test. 
    :type face: bmesh.types.BMFace
    :param point: The point to test. 
    :type point: float triplet
    :return:  True when the projection of the point is in the face. 
    '''

    pass
